using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BOE.BOECommon;
using CrystalDecisions.Enterprise;
using CrystalDecisions.Enterprise.Desktop;


namespace BOE.BOEUserAudit
{
  public partial class BOEUserAuditForm : Form
  {
    private bool cmsLoggedOn = false;
    private bool dbLoggedOn = false;
    private BOECommon.BOECommon _common = null;
    private SqlConnection dbConnection = null;
    private bool bCancel = false;
    private const string C_AUDIT_QUERY = 
        "SELECT DISTINCT AUDIT_EVENT.User_Name, max(AUDIT_EVENT.Start_Timestamp) " +
        "FROM AUDIT_EVENT " +
        "WHERE AUDIT_EVENT.User_Name Is Not Null " +
        "  AND AUDIT_EVENT.User_Name != 'Administrator'" +
        "  AND ltrim(rtrim(AUDIT_EVENT.User_Name)) != ''" +
        "GROUP BY AUDIT_EVENT.User_Name ORDER BY AUDIT_EVENT.User_Name";
    
    public BOEUserAuditForm()
    {
      InitializeComponent();
    }

    private void BOEUserAuditForm_FormClosing(object sender, FormClosingEventArgs e)
    {
      if (_common != null)
        _common.Dispose();
      if ((dbConnection != null) && (dbConnection.State == ConnectionState.Open))
        dbConnection.Close();
    }

    private void cmsLogon_Click(object sender, EventArgs e)
    {
      try
      {
        Cursor = Cursors.WaitCursor;
        cmsStatus.Text = "Logging on to CMS...";
        cmsStatus.Refresh();
        //log on to the CMS
        try
        {
          _common = new BOECommon.BOECommon(cmsUser.Text, cmsPassword.Text, cms.Text);
          if (_common.ErrMsg != "")
          {
            cmsStatus.Text = "Unable to log on to CMS: " + _common.ErrMsg;
          }
          else
            cmsStatus.Text = "Logged on to CMS";
          cmsStatus.Refresh();
          cmsLoggedOn = true;

        }
        catch (Exception ex)
        {
          MessageBox.Show("Unable to log on to CMS: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
          cmsLoggedOn = false;
        }

        //now log on to the Audit Database
        if (cmsLoggedOn)
        {
          try
          {
            string connect = "Data Source=SQLWH;Initial Catalog=boe_audit;Integrated Security=False;User=boe;Password=b03_cms";
            dbConnection = new SqlConnection(connect);
            dbConnection.Open();
            dbLoggedOn = (dbConnection.State == ConnectionState.Open);
          }
          catch (Exception ex)
          {
            MessageBox.Show("Unable to log on to Audit Database: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            dbLoggedOn = false;
          }
        }
      }
      finally
      {
        Cursor = Cursors.Default;
        auditBtn.Enabled = cmsLoggedOn;
        cancelBtn.Enabled = cmsLoggedOn;
      }
    }

    private void auditBtn_Click(object sender, EventArgs e)
    {
      Cursor = Cursors.WaitCursor;
      try
      {
        dsAuditInfo.Tables["userTable"].Clear();
        dsAuditInfo.Tables["userGroupTable"].Clear();
        dsAuditInfo.Tables["reportTable"].Clear();
        
        statusLabel.Text = "Processing Business Objects User Data";
        statusLabel.Refresh();
        
        //get user list from BusinessObjects
        //NOTE:  An InfoStore query will only return up to 1000 objects.  If you have more than that
        //       you'll need to set a where clause to load, for example, user names starting with 'A' through 'M' and
        //       then run the query again for user names starting with 'N' through 'Z'

        string query = "Select SI_NAME, SI_USERFULLNAME, SI_ID, SI_NAMEDUSER, SI_LASTLOGONTIME, SI_DESCRIPTION From CI_SYSTEMOBJECTS " +
                       "Where SI_KIND='User' and SI_NAME != 'Administrator' and SI_NAME != 'Guest' " +
                       " and SI_NAME != 'PMUser' and SI_NAME != 'QaaWSServletPrincipal' " +
                       "Order by SI_NAME";

        //We use the "using" statement because it has better memory management than running the query and then using a "for" statement.
        using (InfoObjects users = _common.BOEInfoStore.Query(query))
        {
          for (int i = 1; i <= users.Count; i++)
          {
            if (!bCancel)
            {
              using (User userData = (User)users[i])
              {
                loadUserInfo(userData);
              }

              if ((i % 10) == 0)
              {
                Application.DoEvents();
              }
            }
          } //users.count
          userLabel.Text = string.Empty;
          userLabel.Refresh();
        } //using users
        
        if (!bCancel && dbLoggedOn)
        {
          loadLastActivity();
        }

        //load reports and the users they're assigned to if we're reporting on reports
        if (!bCancel && (rptTree.Nodes[3].Checked || rptTree.Nodes[3].Nodes[0].Checked ||
            rptTree.Nodes[3].Nodes[1].Checked || rptTree.Nodes[1].Nodes[1].Checked ||
            rptTree.Nodes[2].Nodes[1].Checked))
        {
          statusLabel.Text = "Processing Business Objects Report Data";
          statusLabel.Refresh();
          loadReportRights();
        }

        statusLabel.Text = "Generating Report";
        statusLabel.Refresh();
        RunReport();
        statusLabel.Text = "Done.";
        statusLabel.Refresh();
        showRptBtn.Enabled = true;
      }
      finally
      {
        Cursor = Cursors.Default;
      }
    }

    private void showRptBtn_Click(object sender, EventArgs e)
    {
      RunReport();
    }

    private void cancelBtn_Click(object sender, EventArgs e)
    {
      bCancel = (MessageBox.Show("Cancel Audit?", "Cancel?", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes);
    }

    private void loadUserInfo(User userData)
    {
      bool namedUser = (userData.Connection == CeConnectionType.ceConnectionNamed);

      AddUserRow(userData.Title, userData.FullName, namedUser, userData.Description);

      //get groups user is assigned to
      string query = "Select SI_ID, SI_USERGROUPS from CI_SYSTEMOBJECTS where SI_ID=" + userData.ID.ToString();
      using (InfoObjects userInfo = _common.BOEInfoStore.Query(query))
      {
        if (userInfo.Count > 0)
          AddGroupRow(userInfo[1]);
      }
    }

    private void AddUserRow(string userID, string userName, bool namedUser, string descrip)
    {
      userLabel.Text = userID;
      userLabel.Refresh();
      string uname = string.Empty;
      if (userName != string.Empty)
        uname = userName;
      else
        uname = userID;
      dsAuditInfo.userTable.AdduserTableRow(userID, uname, namedUser, descrip);
    }

    private void AddGroupRow(InfoObject userInfo)
    {
      User user = (User)userInfo;
      Groups userGroups = user.Groups;

      string query = "Select SI_NAME From CI_SYSTEMOBJECTS Where SI_ID=:: and SI_NAME != 'Everyone' and SI_NAME != 'Universe Designer Users'";

      foreach (int groupID in userGroups)
      {
        using (InfoObjects groupInfo = _common.BOEInfoStore.Query(query.Replace("::", groupID.ToString())))
        {
          if (groupInfo.Count > 0)
          {
            dsAuditInfo.userGroupTable.AdduserGroupTableRow(user.Title, groupInfo[1].Title);
          }
        }
      }
    }

    private void loadReportRights()
    {
      //NOTE:  An InfoStore query will only return up to 1000 objects.  If you have more than that
      //       you'll need to set a where clause to load, for example, report names starting with 'A' through 'M' and
      //       then run the query again for report names starting with 'N' through 'Z'
      string query = "Select SI_ID, SI_NAME, SI_PARENTID, SI_KIND From CI_INFOOBJECTS " +
                     "Where SI_KIND in ('CrystalReport','Webi','Excel','PDF','Rtf','Txt','Universe','Word') " +
                     "  and SI_INSTANCE_OBJECT<1 " +
                     "order by SI_NAME, SI_ID";
      using (InfoObjects rpts = _common.BOEInfoStore.Query(query))
      {
        for (int i = 1; i <= rpts.Count; i++)
        {
          using (InfoObject rpt = rpts[i])
          {
            userLabel.Text = rpt.ID.ToString() + "-" + rpt.Title;
            userLabel.Refresh();
            dsAuditInfo.reportTable.AddreportTableRow(rpt.ID, rpt.Title, rpt.Kind, _common.GetPathFromFolderID(rpt.ParentID));
            string role;
            using (SecurityInfo si = rpt.SecurityInfo)
            {
              for (int j = 1; j <= si.ObjectPrincipals.Count; j++)
              {
                if (si.ObjectPrincipals[j].Role != CeRole.ceRoleNoAccess)
                {
                  switch (si.ObjectPrincipals[j].Role)
                  {
                    case CeRole.ceRoleAdvanced:
                      role = "Advanced";
                      break;
                    case CeRole.ceRoleFullControl:
                      role = "Full Control";
                      break;
                    case CeRole.ceRoleSchedule:
                      role = "Schedule";
                      break;
                    case CeRole.ceRoleView:
                      role = "View";
                      break;
                    case CeRole.ceRoleViewOnDemand:
                      role = "View on Demand";
                      break;
                    default:
                      role = string.Empty;
                      break;
                  }
                  try
                  {
                    dsAuditInfo.reportGroupTable.AddreportGroupTableRow(rpt.ID, si.ObjectPrincipals[j].Name, role);
                  }
                  catch (Exception ex)
                  {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                  }
                  if ((i % 20) == 0)
                  {
                    Application.DoEvents();
                  }
                } //if role
              } //for j
            } //using si
          } //using rpt
        } //for i
      } //using rpts
    }

    private void RunReport()
    {
      //Dept List
      if (rptTree.Nodes[0].Checked)
      {
        BOEAuditRptViewer deptViewer = new BOEAuditRptViewer(dsAuditInfo, BOEAuditRptViewer.AuditRptType.UserList);
        deptViewer.Show();
      }
      //User Reports
      if (rptTree.Nodes[1].Checked || rptTree.Nodes[1].Nodes[0].Checked)
      {
        //User Groups report
        BOEAuditRptViewer viewer = new BOEAuditRptViewer(dsAuditInfo, BOEAuditRptViewer.AuditRptType.UserGroups);
        viewer.Show();
      }
      if (rptTree.Nodes[1].Checked || rptTree.Nodes[1].Nodes[1].Checked)
      {
        //run User Reports report
        BOEAuditRptViewer uRptViewer = new BOEAuditRptViewer(dsAuditInfo, BOEAuditRptViewer.AuditRptType.UserReports);
        uRptViewer.Show();
      }

      //Group reports
      if (rptTree.Nodes[2].Checked || rptTree.Nodes[2].Nodes[0].Checked)
      {
        //Group with Users
        BOEAuditRptViewer grpViewer = new BOEAuditRptViewer(dsAuditInfo, BOEAuditRptViewer.AuditRptType.GroupUsers);
        grpViewer.Show();
      }

      if (rptTree.Nodes[2].Checked || rptTree.Nodes[2].Nodes[1].Checked)
      {
        //Group with Reports
        BOEAuditRptViewer grpRptViewer = new BOEAuditRptViewer(dsAuditInfo, BOEAuditRptViewer.AuditRptType.GroupReports);
        grpRptViewer.Show();
      }

      //Reports reports
      if (rptTree.Nodes[3].Checked || rptTree.Nodes[3].Nodes[0].Checked)
      {
        //Reports with groups
        BOEAuditRptViewer rptViewer = new BOEAuditRptViewer(dsAuditInfo, BOEAuditRptViewer.AuditRptType.ReportGroups);
        rptViewer.Show();
      }

      if (rptTree.Nodes[3].Checked || rptTree.Nodes[3].Nodes[1].Checked)
      {
        //Reports with Users
        MessageBox.Show("Report Not Yet Implemented");
      }     
    }

    private void loadLastActivity()
    {
      int rows = 0;
      SqlCommand cmd = new SqlCommand(C_AUDIT_QUERY, dbConnection);
      SqlDataReader rdr = cmd.ExecuteReader();
      try
      {
        string userName;
        DateTime activityDate;
        while (!bCancel && rdr.Read())
        {
          userName = rdr[0].ToString();
          activityDate = rdr.GetDateTime(1);

          dsAuditInfo.lastActivity.AddlastActivityRow(userName, activityDate);
          rows++;
          if ((rows % 20) == 0)
          {
            Application.DoEvents();
          }
        }
      }
      finally
      {
        rdr.Close();
      }

    }

  }
}